{% extends "users/layout.html" %}
{% block users_title %}{{ _("Reset password") }}{% endblock %}
{% block users_content %}
<{ form_tag form id="reset-password-form" }>
    <{ form_field form.password }/>
    <{ form_field form.confirm_password }/>
    <{ form_btn _("Reset password") }/>
</{}>
{% endblock %}
